@@ -161,8 +161,8 @@ class Generator:
 
     def generate(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
-        specifications) that represent the complete presentatin for the
-        object.  The presentatin to be generated depends highly upon the
+        specifications) that represent the complete presentation for the
+        object.  The presentation to be generated depends highly upon the
         formatting strings in formatting.py.
 
         args is a dictionary that may contain any of the following:
@@ -181,6 +181,12 @@ class Generator:
         - forceTutorial: boolean (default=False) which says if we
           should force a tutorial to be spoken or not
         """
+
+        if self._script.utilities.isDead(obj):
+            msg = 'ERROR: Cannot generate presentation dead obj'
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return []
+
         startTime = time.time()
         result = []
         globalsDict = {}
@@ -1006,7 +1012,7 @@ class Generator:
 
     def _generateApplicationName(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
-        represents the name of the applicaton for the object.
+        represents the name of the application for the object.
         """
         result = []
         try:
@@ -1077,7 +1083,12 @@ class Generator:
         """
         result = []
         rad = self._script.utilities.realActiveDescendant(obj)
-        return self._generateDisplayedText(rad, **args)
+
+        if not (rad.getRole() == pyatspi.ROLE_TABLE_CELL and rad.childCount):
+            return self._generateDisplayedText(rad, **args)
+
+        content = [self._script.utilities.displayedText(x).strip() for x in rad]
+        return [" ".join(filter(lambda x: x, content))]
 
     def _generateRealActiveDescendantRoleName(self, obj, **args ):
         """Objects, such as tables and trees, can represent individual cells
@@ -1164,10 +1175,6 @@ class Generator:
 
     def _getAlternativeRole(self, obj, **args):
         if self._script.utilities.isMath(obj):
-            if self._script.utilities.isMathFraction(obj):
-                return 'ROLE_MATH_FRACTION'
-            if self._script.utilities.isMathRoot(obj):
-                return 'ROLE_MATH_ROOT'
             if self._script.utilities.isMathSubOrSuperScript(obj):
                 return 'ROLE_MATH_SCRIPT_SUBSUPER'
             if self._script.utilities.isMathUnderOrOverScript(obj):
@@ -1182,10 +1189,17 @@ class Generator:
                 return 'ROLE_MATH_TABLE'
             if self._script.utilities.isMathTableRow(obj):
                 return 'ROLE_MATH_TABLE_ROW'
-        if self._script.utilities.isStatic(obj):
-            return 'ROLE_STATIC'
+        if self._script.utilities.isDPub(obj):
+            if self._script.utilities.isDPubFootnote(obj):
+                return 'ROLE_FOOTNOTE'
+            if self._script.utilities.isLandmark(obj):
+                return 'ROLE_DPUB_LANDMARK'
+            if obj.getRole() == pyatspi.ROLE_SECTION:
+                return 'ROLE_DPUB_SECTION'
         if self._script.utilities.isSwitch(obj):
             return 'ROLE_SWITCH'
+        if self._script.utilities.isAnchor(obj):
+            return pyatspi.ROLE_STATIC
         if self._script.utilities.isBlockquote(obj):
             return pyatspi.ROLE_BLOCK_QUOTE
         if self._script.utilities.isLandmark(obj):
@@ -1227,12 +1241,90 @@ class Generator:
                 if isVertical:
                     return object_properties.ROLE_SPLITTER_HORIZONTAL
 
+        if self._script.utilities.isFeed(obj):
+            return object_properties.ROLE_FEED
+
+        if self._script.utilities.isFigure(obj):
+            return object_properties.ROLE_FIGURE
+
         if self._script.utilities.isMenuButton(obj):
             return object_properties.ROLE_MENU_BUTTON
 
         if self._script.utilities.isSwitch(obj):
             return object_properties.ROLE_SWITCH
 
+        if self._script.utilities.isDPub(obj):
+            if self._script.utilities.isLandmark(obj):
+                if self._script.utilities.isDPubAcknowledgments(obj):
+                    return object_properties.ROLE_ACKNOWLEDGMENTS
+                if self._script.utilities.isDPubAfterword(obj):
+                    return object_properties.ROLE_AFTERWORD
+                if self._script.utilities.isDPubAppendix(obj):
+                    return object_properties.ROLE_APPENDIX
+                if self._script.utilities.isDPubBibliography(obj):
+                    return object_properties.ROLE_BIBLIOGRAPHY
+                if self._script.utilities.isDPubChapter(obj):
+                    return object_properties.ROLE_CHAPTER
+                if self._script.utilities.isDPubConclusion(obj):
+                    return object_properties.ROLE_CONCLUSION
+                if self._script.utilities.isDPubCredits(obj):
+                    return object_properties.ROLE_CREDITS
+                if self._script.utilities.isDPubEndnotes(obj):
+                    return object_properties.ROLE_ENDNOTES
+                if self._script.utilities.isDPubEpilogue(obj):
+                    return object_properties.ROLE_EPILOGUE
+                if self._script.utilities.isDPubErrata(obj):
+                    return object_properties.ROLE_ERRATA
+                if self._script.utilities.isDPubForeword(obj):
+                    return object_properties.ROLE_FOREWORD
+                if self._script.utilities.isDPubGlossary(obj):
+                    return object_properties.ROLE_GLOSSARY
+                if self._script.utilities.isDPubIndex(obj):
+                    return object_properties.ROLE_INDEX
+                if self._script.utilities.isDPubIntroduction(obj):
+                    return object_properties.ROLE_INTRODUCTION
+                if self._script.utilities.isDPubPagelist(obj):
+                    return object_properties.ROLE_PAGELIST
+                if self._script.utilities.isDPubPart(obj):
+                    return object_properties.ROLE_PART
+                if self._script.utilities.isDPubPreface(obj):
+                    return object_properties.ROLE_PREFACE
+                if self._script.utilities.isDPubPrologue(obj):
+                    return object_properties.ROLE_PROLOGUE
+                if self._script.utilities.isDPubToc(obj):
+                    return object_properties.ROLE_TOC
+            elif role == "ROLE_DPUB_SECTION":
+                if self._script.utilities.isDPubAbstract(obj):
+                    return object_properties.ROLE_ABSTRACT
+                if self._script.utilities.isDPubColophon(obj):
+                    return object_properties.ROLE_COLOPHON
+                if self._script.utilities.isDPubCredit(obj):
+                    return object_properties.ROLE_CREDIT
+                if self._script.utilities.isDPubDedication(obj):
+                    return object_properties.ROLE_DEDICATION
+                if self._script.utilities.isDPubEpigraph(obj):
+                    return object_properties.ROLE_EPIGRAPH
+                if self._script.utilities.isDPubExample(obj):
+                    return object_properties.ROLE_EXAMPLE
+                if self._script.utilities.isDPubPullquote(obj):
+                    return object_properties.ROLE_PULLQUOTE
+                if self._script.utilities.isDPubQna(obj):
+                    return object_properties.ROLE_QNA
+            elif role == pyatspi.ROLE_LIST_ITEM:
+                if self._script.utilities.isDPubBiblioentry(obj):
+                    return object_properties.ROLE_BIBLIOENTRY
+                if self._script.utilities.isDPubEndnote(obj):
+                    return object_properties.ROLE_ENDNOTE
+            else:
+                if self._script.utilities.isDPubCover(obj):
+                    return object_properties.ROLE_COVER
+                if self._script.utilities.isDPubFootnote(obj):
+                    return object_properties.ROLE_FOOTNOTE
+                if self._script.utilities.isDPubPagebreak(obj):
+                    return object_properties.ROLE_PAGEBREAK
+                if self._script.utilities.isDPubSubtitle(obj):
+                    return object_properties.ROLE_SUBTITLE
+
         if self._script.utilities.isLandmark(obj):
             if self._script.utilities.isLandmarkBanner(obj):
                 return object_properties.ROLE_LANDMARK_BANNER
