@@ -85,12 +85,6 @@ _scriptManager = script_manager.getManager()
 _settingsManager = settings_manager.getManager()
 _logger = logger.getLogger()
 
-try:
-    # If we don't have an active desktop, we will get a RuntimeError.
-    from . import mouse_review
-except RuntimeError:
-    pass
-
 def onEnabledChanged(gsetting, key):
     try:
         enabled = gsetting.get_boolean(key)
@@ -386,13 +380,6 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     if _settingsManager.getSetting('enableSound'):
         player.init()
 
-    # I'm not sure where else this should go. But it doesn't really look
-    # right here.
-    try:
-        mouse_review.mouse_reviewer.toggle(on=settings.enableMouseReview)
-    except NameError:
-        pass
-
     global _orcaModifiers
     custom = [k for k in settings.orcaModifierKeys if k not in _orcaModifiers]
     _orcaModifiers += custom
@@ -432,7 +419,7 @@ def _showPreferencesUI(script, prefs):
     orca_state.orcaOS.showGUI()
 
 def showAppPreferencesGUI(script=None, inputEvent=None):
-    """Displays the user interace to configure the settings for a
+    """Displays the user interface to configure the settings for a
     specific applications within Orca and set up those app-specific
     user preferences using a GUI.
 
@@ -449,7 +436,7 @@ def showAppPreferencesGUI(script=None, inputEvent=None):
     return True
 
 def showPreferencesGUI(script=None, inputEvent=None):
-    """Displays the user interace to configure Orca and set up
+    """Displays the user interface to configure Orca and set up
     user preferences using a GUI.
 
     Returns True to indicate the input event has been consumed.
@@ -487,7 +474,7 @@ def quitOrca(script=None, inputEvent=None):
     return True
 
 def showFindGUI(script=None, inputEvent=None):
-    """Displays the user interace to perform an Orca Find.
+    """Displays the user interface to perform an Orca Find.
 
     Returns True to indicate the input event has been consumed.
     """
