@@ -698,14 +698,15 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source)
             return
 
-        if self.utilities.isZombie(event.source):
+        role = event.source.getRole()
+
+        if self.utilities.isZombie(event.source) \
+           or role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_LIST]:
             comboBox = self.utilities.containingComboBox(event.source)
             if comboBox:
                 orca.setLocusOfFocus(event, comboBox, True)
                 return
 
-        role = event.source.getRole()
-
         # This seems to be something we inherit from Gtk+
         if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_PASSWORD_TEXT]:
             orca.setLocusOfFocus(event, event.source)
@@ -716,6 +717,11 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source)
             return
 
+        # Ditto.
+        if role == pyatspi.ROLE_TOGGLE_BUTTON:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
         # Ditto.
         if role == pyatspi.ROLE_COMBO_BOX:
             orca.setLocusOfFocus(event, event.source)
@@ -782,10 +788,6 @@ class Script(default.Script):
                 orca.setLocusOfFocus(event, event.source, False)
                 return
 
-        # We should present this in response to active-descendant-changed events
-        if event.source.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS):
-            return
-
         default.Script.onFocusedChanged(self, event)
 
     def onCaretMoved(self, event):
