@@ -734,44 +734,11 @@ class SpeechGenerator(generator.Generator):
         dictionary.  The 'priorObj' is typically set by Orca to be the
         previous object with focus.
         """
-        result = []
-        acss = self.voice(DEFAULT)
-        if obj:
-            priorObj = args.get('priorObj', None)
-            try:
-                priorParent = priorObj.parent
-            except:
-                priorParent = None
 
-            if (obj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                or (obj.parent and obj.parent.getRole() == pyatspi.ROLE_TABLE):
-                try:
-                    table = priorParent.queryTable()
-                except:
-                    table = None
-                if table \
-                   and ((priorObj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                         or (priorObj.getRole() == pyatspi.ROLE_TABLE)):
-                    index = self._script.utilities.cellIndex(priorObj)
-                    oldRow = table.getRowAtIndex(index)
-                else:
-                    oldRow = -1
+        if not self._script.utilities.cellRowChanged(obj):
+            return []
 
-                try:
-                    table = obj.parent.queryTable()
-                except:
-                    pass
-                else:
-                    index = self._script.utilities.cellIndex(obj)
-                    newRow = table.getRowAtIndex(index)
-                    if (newRow >= 0) \
-                       and (index != newRow) \
-                       and ((newRow != oldRow) \
-                            or (obj.parent != priorParent)):
-                        result = self._generateRowHeader(obj, **args)
-        if result:
-            result.extend(acss)
-        return result
+        return self._generateRowHeader(obj, **args)
 
     def _generateNewColumnHeader(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
@@ -783,44 +750,14 @@ class SpeechGenerator(generator.Generator):
         dictionary.  The 'priorObj' is typically set by Orca to be the
         previous object with focus.
         """
-        result = []
-        acss = self.voice(DEFAULT)
-        if obj and not args.get('readingRow', False):
-            priorObj = args.get('priorObj', None)
-            try:
-                priorParent = priorObj.parent
-            except:
-                priorParent = None
 
-            if (obj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                or (obj.parent and obj.parent.getRole() == pyatspi.ROLE_TABLE):
-                try:
-                    table = priorParent.queryTable()
-                except:
-                    table = None
-                if table \
-                   and ((priorObj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                         or (priorObj.getRole() == pyatspi.ROLE_TABLE)):
-                    index = self._script.utilities.cellIndex(priorObj)
-                    oldCol = table.getColumnAtIndex(index)
-                else:
-                    oldCol = -1
+        if not self._script.utilities.cellColumnChanged(obj):
+            return []
 
-                try:
-                    table = obj.parent.queryTable()
-                except:
-                    pass
-                else:
-                    index = self._script.utilities.cellIndex(obj)
-                    newCol = table.getColumnAtIndex(index)
-                    if (newCol >= 0) \
-                       and (index != newCol) \
-                       and ((newCol != oldCol) \
-                            or (obj.parent != priorParent)):
-                        result = self._generateColumnHeader(obj, **args)
-        if result:
-            result.extend(acss)
-        return result
+        if args.get('readingRow'):
+            return []
+
+        return self._generateColumnHeader(obj, **args)
 
     def _generateRealTableCell(self, obj, **args):
         """Orca has a feature to automatically read an entire row of a table
@@ -1400,6 +1337,18 @@ class SpeechGenerator(generator.Generator):
         if children:
             result.append(messages.itemCount(children))
             result.extend(acss)
+            return result
+
+        role = args.get('role', obj.getRole())
+        if role in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
+            children = [x for x in obj if x.getRole() == pyatspi.ROLE_LIST_ITEM]
+            setsize = len(children)
+            if not setsize:
+                return []
+
+            result = [messages.listItemCount(setsize)]
+            result.extend(acss)
+
         return result
 
     def _generateNoShowingChildren(self, obj, **args):
@@ -1540,6 +1489,8 @@ class SpeechGenerator(generator.Generator):
         allRoles = [pyatspi.ROLE_BLOCK_QUOTE,
                     pyatspi.ROLE_FORM,
                     pyatspi.ROLE_LANDMARK,
+                    'ROLE_DPUB_LANDMARK',
+                    'ROLE_DPUB_SECTION',
                     pyatspi.ROLE_LIST,
                     pyatspi.ROLE_PANEL,
                     pyatspi.ROLE_TABLE]
@@ -1549,11 +1500,11 @@ class SpeechGenerator(generator.Generator):
             if _settingsManager.getSetting('sayAllContextBlockquote'):
                 enabled.append(pyatspi.ROLE_BLOCK_QUOTE)
             if _settingsManager.getSetting('sayAllContextLandmark'):
-                enabled.append(pyatspi.ROLE_LANDMARK)
+                enabled.extend([pyatspi.ROLE_LANDMARK, 'ROLE_DPUB_LANDMARK'])
             if _settingsManager.getSetting('sayAllContextList'):
                 enabled.append(pyatspi.ROLE_LIST)
             if _settingsManager.getSetting('sayAllContextPanel'):
-                enabled.append(pyatspi.ROLE_PANEL)
+                enabled.extend([pyatspi.ROLE_PANEL, 'ROLE_DPUB_SECTION'])
             if _settingsManager.getSetting('sayAllContextNonLandmarkForm'):
                 enabled.append(pyatspi.ROLE_FORM)
             if _settingsManager.getSetting('sayAllContextTable'):
@@ -1562,11 +1513,11 @@ class SpeechGenerator(generator.Generator):
             if _settingsManager.getSetting('speakContextBlockquote'):
                 enabled.append(pyatspi.ROLE_BLOCK_QUOTE)
             if _settingsManager.getSetting('speakContextLandmark'):
-                enabled.append(pyatspi.ROLE_LANDMARK)
+                enabled.extend([pyatspi.ROLE_LANDMARK, 'ROLE_DPUB_LANDMARK'])
             if _settingsManager.getSetting('speakContextList'):
                 enabled.append(pyatspi.ROLE_LIST)
             if _settingsManager.getSetting('speakContextPanel'):
-                enabled.append(pyatspi.ROLE_PANEL)
+                enabled.extend([pyatspi.ROLE_PANEL, 'ROLE_DPUB_SECTION'])
             if _settingsManager.getSetting('speakContextNonLandmarkForm'):
                 enabled.append(pyatspi.ROLE_FORM)
             if _settingsManager.getSetting('speakContextTable'):
@@ -1597,26 +1548,87 @@ class SpeechGenerator(generator.Generator):
                 result.append(messages.leavingNLists(count))
             else:
                 result.append(messages.LEAVING_LIST)
-        elif role == pyatspi.ROLE_PANEL and self._script.utilities.isDocumentPanel(obj):
-            result.append(messages.LEAVING_PANEL)
+        elif role == pyatspi.ROLE_PANEL:
+            if self._script.utilities.isFeed(obj):
+                result.append(messages.LEAVING_FEED)
+            elif self._script.utilities.isFigure(obj):
+                result.append(messages.LEAVING_FIGURE)
+            elif self._script.utilities.isDocumentPanel(obj):
+                result.append(messages.LEAVING_PANEL)
         elif role == pyatspi.ROLE_TABLE and self._script.utilities.isTextDocumentTable(obj):
             result.append(messages.LEAVING_TABLE)
+        elif role == 'ROLE_DPUB_LANDMARK':
+            if self._script.utilities.isDPubAcknowledgments(obj):
+                result.append(messages.LEAVING_ACKNOWLEDGMENTS)
+            elif self._script.utilities.isDPubAfterword(obj):
+                result.append(messages.LEAVING_AFTERWORD)
+            elif self._script.utilities.isDPubAppendix(obj):
+                result.append(messages.LEAVING_APPENDIX)
+            elif self._script.utilities.isDPubBibliography(obj):
+                result.append(messages.LEAVING_BIBLIOGRAPHY)
+            elif self._script.utilities.isDPubChapter(obj):
+                result.append(messages.LEAVING_CHAPTER)
+            elif self._script.utilities.isDPubConclusion(obj):
+                result.append(messages.LEAVING_CONCLUSION)
+            elif self._script.utilities.isDPubCredits(obj):
+                result.append(messages.LEAVING_CREDITS)
+            elif self._script.utilities.isDPubEndnotes(obj):
+                result.append(messages.LEAVING_ENDNOTES)
+            elif self._script.utilities.isDPubEpilogue(obj):
+                result.append(messages.LEAVING_EPILOGUE)
+            elif self._script.utilities.isDPubErrata(obj):
+                result.append(messages.LEAVING_ERRATA)
+            elif self._script.utilities.isDPubForeword(obj):
+                result.append(messages.LEAVING_FOREWORD)
+            elif self._script.utilities.isDPubGlossary(obj):
+                result.append(messages.LEAVING_GLOSSARY)
+            elif self._script.utilities.isDPubIndex(obj):
+                result.append(messages.LEAVING_INDEX)
+            elif self._script.utilities.isDPubIntroduction(obj):
+                result.append(messages.LEAVING_INTRODUCTION)
+            elif self._script.utilities.isDPubPagelist(obj):
+                result.append(messages.LEAVING_PAGELIST)
+            elif self._script.utilities.isDPubPart(obj):
+                result.append(messages.LEAVING_PART)
+            elif self._script.utilities.isDPubPreface(obj):
+                result.append(messages.LEAVING_PREFACE)
+            elif self._script.utilities.isDPubPrologue(obj):
+                result.append(messages.LEAVING_PROLOGUE)
+            elif self._script.utilities.isDPubToc(obj):
+                result.append(messages.LEAVING_TOC)
+        elif role == 'ROLE_DPUB_SECTION':
+            if self._script.utilities.isDPubAbstract(obj):
+                result.append(messages.LEAVING_ABSTRACT)
+            elif self._script.utilities.isDPubColophon(obj):
+                result.append(messages.LEAVING_COLOPHON)
+            elif self._script.utilities.isDPubCredit(obj):
+                result.append(messages.LEAVING_CREDIT)
+            elif self._script.utilities.isDPubDedication(obj):
+                result.append(messages.LEAVING_DEDICATION)
+            elif self._script.utilities.isDPubEpigraph(obj):
+                result.append(messages.LEAVING_EPIGRAPH)
+            elif self._script.utilities.isDPubExample(obj):
+                result.append(messages.LEAVING_EXAMPLE)
+            elif self._script.utilities.isDPubPullquote(obj):
+                result.append(messages.LEAVING_PULLQUOTE)
+            elif self._script.utilities.isDPubQna(obj):
+                result.append(messages.LEAVING_QNA)
         elif self._script.utilities.isLandmark(obj):
             if self._script.utilities.isLandmarkBanner(obj):
                 result.append(messages.LEAVING_LANDMARK_BANNER)
-            if self._script.utilities.isLandmarkComplementary(obj):
+            elif self._script.utilities.isLandmarkComplementary(obj):
                 result.append(messages.LEAVING_LANDMARK_COMPLEMENTARY)
-            if self._script.utilities.isLandmarkContentInfo(obj):
+            elif self._script.utilities.isLandmarkContentInfo(obj):
                 result.append(messages.LEAVING_LANDMARK_CONTENTINFO)
-            if self._script.utilities.isLandmarkMain(obj):
+            elif self._script.utilities.isLandmarkMain(obj):
                 result.append(messages.LEAVING_LANDMARK_MAIN)
-            if self._script.utilities.isLandmarkNavigation(obj):
+            elif self._script.utilities.isLandmarkNavigation(obj):
                 result.append(messages.LEAVING_LANDMARK_NAVIGATION)
-            if self._script.utilities.isLandmarkRegion(obj):
+            elif self._script.utilities.isLandmarkRegion(obj):
                 result.append(messages.LEAVING_LANDMARK_REGION)
-            if self._script.utilities.isLandmarkSearch(obj):
+            elif self._script.utilities.isLandmarkSearch(obj):
                 result.append(messages.LEAVING_LANDMARK_SEARCH)
-            if self._script.utilities.isLandmarkForm(obj):
+            elif self._script.utilities.isLandmarkForm(obj):
                 result.append(messages.LEAVING_FORM)
         elif role == pyatspi.ROLE_FORM:
             result.append(messages.LEAVING_FORM)
@@ -1668,10 +1680,21 @@ class SpeechGenerator(generator.Generator):
         skipRoles.extend(disabled)
 
         stopAtRoles = args.get('stopAtRoles', [])
-        stopAtRoles.append(pyatspi.ROLE_APPLICATION)
+        stopAtRoles.extend([pyatspi.ROLE_APPLICATION, pyatspi.ROLE_MENU_BAR])
 
         presentOnce = [pyatspi.ROLE_BLOCK_QUOTE, pyatspi.ROLE_LIST]
 
+        presentCommonAncestor = False
+        if commonAncestor and not leaving:
+            commonRole = self._getAlternativeRole(commonAncestor)
+            if commonRole in presentOnce:
+                pred = lambda x: x and self._getAlternativeRole(x) == commonRole
+                objAncestor = pyatspi.findAncestor(obj, pred)
+                priorAncestor = pyatspi.findAncestor(priorObj, pred)
+                objLevel = self._script.utilities.nestingLevel(objAncestor)
+                priorLevel = self._script.utilities.nestingLevel(priorAncestor)
+                presentCommonAncestor = objLevel != priorLevel
+
         ancestors, ancestorRoles = [], []
         parent = obj.parent
         while parent and parent != parent.parent:
@@ -1684,7 +1707,7 @@ class SpeechGenerator(generator.Generator):
                 pass
             elif self._script.utilities.isLayoutOnly(parent):
                 pass
-            elif parent != commonAncestor or (not leaving and parentRole in presentOnce):
+            elif parent != commonAncestor or presentCommonAncestor:
                 ancestors.append(parent)
                 ancestorRoles.append(parentRole)
 
@@ -1732,6 +1755,8 @@ class SpeechGenerator(generator.Generator):
         args['includeOnly'] = [pyatspi.ROLE_BLOCK_QUOTE,
                                pyatspi.ROLE_FORM,
                                pyatspi.ROLE_LANDMARK,
+                               'ROLE_DPUB_LANDMARK',
+                               'ROLE_DPUB_SECTION',
                                pyatspi.ROLE_LIST,
                                pyatspi.ROLE_PANEL,
                                pyatspi.ROLE_TABLE]
@@ -2008,6 +2033,7 @@ class SpeechGenerator(generator.Generator):
                        pyatspi.ROLE_PUSH_BUTTON,
                        pyatspi.ROLE_RADIO_BUTTON,
                        pyatspi.ROLE_SLIDER,
+                       pyatspi.ROLE_TEXT,
                        pyatspi.ROLE_TOGGLE_BUTTON]
         isWidget = lambda x: x and x.getRole() in widgetRoles
         result = []
